<?php

namespace Teknisa\FIS\Factories;

use Doctrine\ORM\EntityManager;

class Ger58000ApuracaoIRPJCSLLFactory {
    
    protected $connection;

    public function __construct(EntityManager $entityManager) {
        $this->connection = $entityManager->getConnection();
    }
    
    public function checkParamImposto() {
        $sql = "SELECT NRSEQPARAMIMPOSTO FROM PARAMIMPOSTO WHERE ROWNUM = 1";
        
        $std = $this->connection->prepare($sql);
        $params = array();
        $std->execute($params);
        
        return $std->fetchAll();
        
    }
    
    public function checkParamapu($NMPARAMAPU, $IDPARAMAPU, $IDIMPOSTO, $NRORG) {
        
        $sql = "SELECT 1 FROM PARAMAPUIRPJ WHERE NMPARAMAPU = :NMPARAMAPU AND IDPARAMAPU = :IDPARAMAPU AND IDIMPOSTO = :IDIMPOSTO AND NRORG = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $params = array('NMPARAMAPU' => $NMPARAMAPU,
                        'IDPARAMAPU' => $IDPARAMAPU,
                        'IDIMPOSTO' => $IDIMPOSTO,
                        'NRORG' => $NRORG );
        $std->execute($params);
        
        return $std->fetch();
        
    }
    
    public function checkAliqIRPJCSLL($DSALIQIRPJCSLL, $NRORG) {
        
        $sql = "SELECT 1 FROM ALIQIRPJCSLL WHERE DSALIQIRPJCSLL = :DSALIQIRPJCSLL AND NRORG = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $params = array('DSALIQIRPJCSLL' => $DSALIQIRPJCSLL,
                        'NRORG' => $NRORG );
        $std->execute($params);
        
        return $std->fetch();
        
    }
    
    public function checkDefaultAliqIRPJCSLL($rows, $nrorg) {
        
        $keysToRemove = array();
        
        
        $sql = "SELECT 1 FROM ALIQIRPJCSLL WHERE DSALIQIRPJCSLL = :DSALIQIRPJCSLL AND NRORG = :NRORG";
        $std = $this->connection->prepare($sql);
        
        //Procura pelas rows com nomes já cadastrados 
        foreach ($rows as $key => $row) {
            $params = array('DSALIQIRPJCSLL' => $row['DSALIQIRPJCSLL'],
                            'NRORG' => $nrorg );
            $std->execute($params);
            if ($std->fetch()) $keysToRemove[] = $key;
        }
        
        //Remove as rows com nomes já cadastrados
        foreach($keysToRemove as $key) {
            unset($rows[$key]);
        }
        
        $rows = array_values($rows);
        
        //Retorna apenas as que devem ser inseridas no banco
        return $rows;
        
    }
    
    public function checkAliqIRPJCSLLRelations($NRSEQALIQIRPJ, $NRORG) {
        
        $sql = "SELECT 1 FROM RECDESPIRPJ WHERE NRSEQALIQIRPJ = :NRSEQALIQIRPJ AND NRORG = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $params = array('NRSEQALIQIRPJ' => $NRSEQALIQIRPJ,
                        'NRORG' => $NRORG );
        $std->execute($params);
        
        return $std->fetch();
        
    }
    
    public function checkRecDespIRPJ($NMRECDESPIRPJ, $IDRECDESPIRPJ, $IDRECDESPIRPJOP, $NRSEQALIQIRPJ, $NRORG) {
        
        $sql = "SELECT 1 FROM RECDESPIRPJ WHERE NMRECDESPIRPJ = :NMRECDESPIRPJ AND IDRECDESPIRPJ = :IDRECDESPIRPJ AND IDRECDESPIRPJOP = :IDRECDESPIRPJOP AND NRSEQALIQIRPJ = :NRSEQALIQIRPJ AND NRORG = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $params = array('NMRECDESPIRPJ' => $NMRECDESPIRPJ,
                        'IDRECDESPIRPJ' => $IDRECDESPIRPJ,
                        'IDRECDESPIRPJOP' => $IDRECDESPIRPJOP,
                        'NRSEQALIQIRPJ' => $NRSEQALIQIRPJ,
                        'NRORG' => $NRORG );
        $std->execute($params);
        
        return $std->fetch();
        
    }
    
    public function checkInfCtbIRPJ($NRSEQPARAMAPU, $CDINFCTBIRPJ, $NRORG) {
        
        $sql = "SELECT 1 FROM PARAMAPUIRPJ P JOIN INFCTBIRPJ I ON P.NRSEQPARAMAPU = I.NRSEQPARAMAPU WHERE I.NRSEQPARAMAPU = :NRSEQPARAMAPU AND I.CDINFCTBIRPJ = :CDINFCTBIRPJ AND I.NRORG = :NRORG";
        $sql2 = "SELECT 1 FROM RECDESPIRPJ R JOIN INFCTBIRPJ I ON R.NRSEQRECDESPIRPJ = I.NRSEQPARAMAPU WHERE I.NRSEQPARAMAPU = :NRSEQPARAMAPU AND I.CDINFCTBIRPJ = :CDINFCTBIRPJ AND I.NRORG = :NRORG";
        $params = array('NRSEQPARAMAPU' => $NRSEQPARAMAPU,
                        'CDINFCTBIRPJ' => $CDINFCTBIRPJ,
                        'NRORG' => $NRORG);
        $std = $this->connection->prepare($sql);
        
        $std->execute($params);
        $existsInParamapu = $std->fetch();
        $std = $this->connection->prepare($sql2);
        $std->execute($params);
        $existsInRecDespIRPJ = $std->fetch();
        
        return $existsInParamapu || $existsInRecDespIRPJ;
        
    }
    
    
}