<?php
namespace Teknisa\FIS\Modules\Acessorias\Controllers;

use Teknisa\FIS\Modules\Acessorias\Services\Fis04800LivroEletronicoService;
use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response\Error;


class Fis04800LivroEletronicoController {

    protected $livroEletronicoService;

    public function __construct(EntityManager $entityManager, Fis04800LivroEletronicoService $livroEletronicoService) {
        $this->entityManager          = $entityManager;
        $this->livroEletronicoService = $livroEletronicoService;
    }

    /*
     * Função para realizar a exportação do SINTEGRA.
     */
    public function exportarLivroEletronico(DTO\Request $request, DTO\Response $response) {
        try {
            
            $row = $request->getRow();
            
            $row["NMARQUIVO"] = str_replace(' ', '', $row["NMARQUIVO"]);

            // Chama a função de exportação.
            $fileName = $this->livroEletronicoService->exportarLivroEletronico($row);

            // Retorna o nome do arquivo exportado.
            $response->addDataset(new DataSet('columnsToReturn', array("fileName" => $fileName)));
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function findAgendamento(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            
            $rows = $this->livroEletronicoService->findAgendamento();
            
            $dataSet = new DataSet('agendobrigacess_fis04800', $rows);
            $response->setDataSets(array($dataSet));
            
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    } 
    
    public function saveAgendamento(DTO\Request $request, DTO\Response $response) {
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            $row = $request->getRow();
            
            // Chama a função de validação.
            $this->livroEletronicoService->saveAgendamento($row);
            
            $this->entityManager->commit();

            // Retorna o nome do arquivo exportado.
            //$response->addDataset(new DataSet('columnsToReturn', array("fileName" => $fileName)));
        } catch(\Exception $e) {
            $this->entityManager->rollback();
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteAgendamento(DTO\Request $request, DTO\Response $response) {
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            $row = $request->getRow();
            
            // Chama a função de validação.
            $this->livroEletronicoService->deleteAgendamento($row);
            
            $this->entityManager->commit();

        } catch(\Exception $e) {
            $this->entityManager->rollback();
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function execAgendamentoLivroEletronico(DTO\Request $request, DTO\Response $response) {
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            $row = $request->getParameters();
            
            $this->livroEletronicoService->execAgendamento($row);
            
            $this->entityManager->commit();

        } catch(\Exception $e) {
            $this->entityManager->rollback();
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
}
