<?php
namespace Teknisa\FIS\Modules\Acessorias\Controllers;

use Doctrine\ORM\EntityManager;
use Teknisa\FIS\Modules\Acessorias\Services\Fis84000DmaBaService;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response\Error;

class Fis84000DmaBaController 
{
    protected $dmaService;
    
    public function __construct(EntityManager $entityManager, Fis84000DmaBaService $dmaService){
        $this->entityManager    = $entityManager;
        $this->dmaService      = $dmaService;
    }
     
    /**
     * Função para realizar a exportação DMA-BA
     */
    public function exportarDma(DTO\Request $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            
            $row["NMARQUIVO"] = str_replace(' ', '', $row["NMARQUIVO"]);
            
            // Chama a função de validação
            $fileName = $this->dmaService->exportarDmaBa($row);
            
            // Retorna o nome do arquivo
            $response->addDataset(new DataSet('columnsToReturn', ["fileName" => $fileName]));
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    } 
    
    public function findAgendamento(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            
            $rows = $this->dmaService->findAgendamento();
            
            $dataSet = new DataSet('agendobrigacess_fis84000', $rows);
            $response->setDataSets(array($dataSet));
            
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    } 
    
    public function saveAgendamento(DTO\Request $request, DTO\Response $response) {
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            $row = $request->getRow();
            
            // Chama a função de validação.
            $fileName = $this->dmaService->saveAgendamento($row);
            
            $this->entityManager->commit();

            // Retorna o nome do arquivo exportado.
            $response->addDataset(new DataSet('columnsToReturn', array("fileName" => $fileName)));
        } catch(\Exception $e) {
            $this->entityManager->rollback();
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteAgendamento(DTO\Request $request, DTO\Response $response) {
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            $row = $request->getRow();
            
            // Chama a função de validação.
            $this->dmaService->deleteAgendamento($row);
            
            $this->entityManager->commit();

        } catch(\Exception $e) {
            $this->entityManager->rollback();
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function execAgendamentoDmaBa(DTO\Request $request, DTO\Response $response) {
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            $row = $request->getParameters();
            
            $this->dmaService->execAgendamento($row);
            
            $this->entityManager->commit();

        } catch(\Exception $e) {
            $this->entityManager->rollback();
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
}