<?php
namespace Teknisa\FIS\Services;

use DateTime;
use Doctrine\ORM\EntityManager;
use Teknisa\FIS\Factories\Fis25000ParametrizacaoContribuicoesFactory;
use Teknisa\Libs\Util\DateUtil;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Row;

class Fis25000ParametrizacaoContribuicoesService {

    //Mensagens
    const ARQUIVO_INVALIDO          = "Operação Bloqueada! O layout do arquivo não está conforme padrão estipulado pela receita federal. Favor baixar o arquivo no site do SPED Contribuições e importar novamente.";
    const ALIQUOTA_PIS_INVALIDA     = "Valor da alíquota do PIS inválida.";
    const ALIQUOTA_COFINS_INVALIDA  = "Valor da alíquota do COFINS inválida.";
    const NATUREZA_RECEITA          = "Natureza da receita deve ser informada.";
    const NATUREZA_RECEITA_INVALIDA = "Natureza da receita só deve ser informada para CST NF-e 04, 05, 06, 07, 08 ou 09.";
    const DTFIM_INF                 = "Data final deve ser informada. Informação já cadastrada para outro intervalo em período posterior.";
    const DTINT_INV                 = "Informação já cadastrada para o intervalo de validade.";
    const DTFIMOLD_INF              = "Informação já cadastrada para intervalo inferior sem data final informada.";
    const CONTA_CONTABIL_SUPERIOR   = "Existe conta contábil de nível superior cadastrada.";
    const CONTA_CONTABIL_CADASTRADA = "Conta contábil já cadastrada.";
    const CONTA_CONTABIL_INFERIOR   = "Existe conta contábil de nível inferior cadastrada.";
    const TAMANHO_TABELA            = array ("439"=>9,"4310"=>9,"4311"=>10,"4312"=>9,"4313"=>7,"4314"=>7,"4315"=>7,"4316"=>7,"4317"=>11) ;

    protected $nrorg;
    protected $operador;
    protected $currentTime;
    protected $environment;
    protected $entityManager;
    protected $dataSourceManager;
    protected $factory;
    protected $newCode;

    public function __construct(Environment $environment, EntityManager $entityManager, $dataSourceManager, Fis25000ParametrizacaoContribuicoesFactory $factory, NewCode $newCode) {
        $this->environment       = $environment;
        $this->entityManager     = $entityManager;
        $this->dataSourceManager = $dataSourceManager;
        $this->factory           = $factory;
        $this->newCode           = $newCode;
        $this->nrorg             = $this->environment->getNrOrg();
        $this->operador          = $this->environment->getCdOperador();
        $this->currentTime       = $this->environment->getCurrentTime()->format("d/m/Y H:i:s");
    }

    public function geraNovoCodigo($contador, $nrorg){
        try {
            return $this->newCode->getCodeByTableName($contador, $nrorg);
        }catch (\Exception $e) {
            throw $e;
        }
    }

    public function salvarImportacao($row){
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();

            $this->factory->deleteCodigoNatureza($this->nrorg, $row['CDTABELASPED']);
            $this->entityManager->flush();

            $this->factory->deleteClassificacaoFiscal($this->nrorg, $row['CDTABELASPED']);
            $this->entityManager->flush();

            $arquivo = $this->retornaArquivo($row['TABELA']);
            $this->salvarCodigoNatureza($arquivo, $row);

            $connection->commit();

        }catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }

    public function retornaArquivo ($tabela){
        try {
            $file = $tabela[0]['b64File'];
            $pos  = strpos($file, ',');
            $file = substr($file,$pos+1);
            $file =  base64_decode($file);
            $file = utf8_encode($file);
            $conteudo = preg_split('/\R/', $file);

            //o arquivo possui no minimo 2 linhas (primeira cabeçalho e o restante os dados)
            if(count($conteudo)>=2){
                //remove cabeçalho
                array_shift($conteudo);
                //retira a ultima linha caso ela esteja em branco
                if( !$conteudo[count($conteudo)-1] ){
                    array_pop($conteudo);
                }
                if(empty($conteudo)){
                    throw new \Exception(self::ARQUIVO_INVALIDO, 1);
                }
                return $conteudo;
            }else{
                throw new \Exception(self::ARQUIVO_INVALIDO, 1);
            }
        }catch (\Exception $e) {
            throw $e;
        }
    }

    public function salvarCodigoNatureza($arquivo,$rowImportacao){
        try {
            $arrayCodigo  = [];
            for ($i = 0; $i<count($arquivo); $i++){
                $ncm    = [];
                $ncm_ex = [];
                $dados  = $this->retornaCamposCodigoNatureza($arquivo[$i],$rowImportacao);

                $row = new Row();
                $row->set('__is_new',true);
                $row->set('NRORG',$this->nrorg);
                $row->set('NRORGULTATU',$this->nrorg);
                $row->set('NRORGINCLUSAO',$this->nrorg);
                $row->set('DTULTATU',$this->currentTime);
                $row->set('DTINCLUSAO',$this->currentTime);
                $row->set('CDOPERINCLUSAO',$this->operador);
                $row->set('CDOPERULTATU',$this->operador);
                $row->set('IDATIVO','S');
                $row->set('CDOPERADOR',$this->operador);
                $row->set('CDTABELASPED',$rowImportacao['CDTABELASPED']);
                $row->set('CDNATUREC',$dados[0]);
                $row->set('DTINIVALID',$dados[2]);
                $row->set('DTFINVALID',$dados[3]);

                //verifica se ja existe a natureza antes de salvar
                $resul = $this->factory->verificaNatureza($row['CDNATUREC'], $row['DTINIVALID'], $row['DTFINVALID'], $row['CDTABELASPED'], $this->nrorg);
                if( empty($resul) ){
                    $dataSetCodigoNatureza = new DataSet('codnaturecspedcontr', [$row]);
                    $this->dataSourceManager->persist($dataSetCodigoNatureza);
                    $this->entityManager->flush();
                }

                if($dados[4]){
                    //salva as classficações fiscais referente a cada natureza
                    $ncm= $this->retornaNcm($dados[4], $dados[0], $rowImportacao['CDTABELASPED'], true);
                    $this->salvarClassificacaoFiscal($ncm);
                }

                if($dados[5]){
                    //remove as classficações fiscais (EXCEÇÕES) referente a cada natureza
                    $ncm_ex   = $this->retornaNcm($dados[5], $dados[0], $rowImportacao['CDTABELASPED'], false);
                    $dataSetNcmEx = new DataSet('codncmnaturec', $ncm_ex);
                    $this->dataSourceManager->delete($dataSetNcmEx);
                    $this->entityManager->flush();
                }
            }

        }catch (\Exception $e) {
            throw $e;
        }
    }

    public function salvarClassificacaoFiscal ($ncm){
        foreach ($ncm as $rowNcm) {
            // verifica se ja exite o ncm antes de inserir;
            $resul = $this->factory->verificaClassificaoFiscal($rowNcm['CDNATUREC'], $rowNcm['CDCLASFISC'], $rowNcm['CDTABELASPED'], $this->nrorg);
            if( empty($resul) ){
                $dataSetNcm = new DataSet('codncmnaturec', [$rowNcm]);
                $this->dataSourceManager->persist($dataSetNcm);
                $this->entityManager->flush();
            }
        }
    }

    // retorna as classificações fiscais por codigo de natureza referente a cada linha do arquivo;
    public function retornaNcm ($ncm, $cdnaturec, $cdtabelasped, $novo){
        $codigoNcm = explode(";", $ncm);
        $arrayNcm  = [];
        for ($i=0; $i< count($codigoNcm); $i++){
            $ncm = $this->factory->buscaClassificaoFiscal($codigoNcm[$i], $this->nrorg);
            foreach ($ncm as $codigo) {
                $row = new Row();
                $row->set('__is_new',$novo);
                $row->set('NRORG',$this->nrorg);
                $row->set('NRORGULTATU',$this->nrorg);
                $row->set('NRORGINCLUSAO',$this->nrorg);
                $row->set('DTULTATU',$this->currentTime);
                $row->set('DTINCLUSAO',$this->currentTime);
                $row->set('CDOPERINCLUSAO',$this->operador);
                $row->set('CDOPERULTATU',$this->operador);
                $row->set('IDATIVO','S');
                $row->set('CDOPERADOR',$this->operador);
                $row->set('CDTABELASPED',$cdtabelasped);
                $row->set('CDNATUREC',$cdnaturec);
                $row->set('CDCLASFISC',$codigo['CDCLASFISC']);

                $arrayNcm [] = $row;
            }
        }

        return $arrayNcm;
    }

    // le cada linha do arquivo de tabelas e retorna os campos que estão nele informados.
    public function retornaCamposCodigoNatureza ($row,$rowImportacao){
        try {
            $campos = explode("|", $row);
            $this->validaArquivo($campos,$rowImportacao);
            $campos[2]= $this->criaData($campos[2]);
            //se existir data final (campo não obrigatório), formata a data informada.
            if($campos[3]){
               $campos[3]= $this->criaData($campos[3]);
            }
            return $campos;
        }catch (\Exception $e) {
            throw $e;
        }
    }

    //valida cada linha do arquivo conforme o padrão criado pela receita federal.
    public function validaArquivo ($campos,$rowImportacao){
        try {
            //verifica se o arquivo possui os  campos necessarios de acordo com a tabela;
            if(count($campos)!== self::TAMANHO_TABELA[$rowImportacao['CDTABELASPED']]){
                throw new \Exception(self::ARQUIVO_INVALIDO, 1);
            }
        }catch (\Exception $e) {
            throw $e;
        }
    }

    public function criaData ($valor){
        try {
            $data = DateTime::createFromFormat('dmY', $valor);
            $data = date_format($data, 'd/m/Y');
            return $data;
        }catch (\Exception $e) {
            throw new \Exception(self::ARQUIVO_INVALIDO, 1);
        }
    }

    public function salvarOutrasContribuicoes($rows){
        try {

            if($rows[0]['__is_new']){
                $rows[0]['CDOUTRECCONT'] = $this->factory->geraCodigOutrasContribuicoes($rows[0]['CDEMPRESA'],$this->nrorg)[0]['CDOUTRECCONT'];
            }
            //valida aliquota PIS que possui o CDIMPOSTO ='50'
            $this->validaAliquotaOutrasContribuicoes($rows[0], $rows[0]['VRALIQPIS'], '50');
            //valida aliquota COFINS que possui o CDIMPOSTO ='60'
            $this->validaAliquotaOutrasContribuicoes($rows[0], $rows[0]['VRALIQCOFINS'], '60');
            $this->validaNaturezaReceitaOutrasContribuicoes($rows[0]);
            $this->validaDatas($rows[0]);

            $dataSet = new DataSet('outreccont', $rows);
            $this->dataSourceManager->persist($dataSet);
        }catch (\Exception $e) {
            throw $e;
        }
    }

    public function salvarContaContabil($rowContaContabil, $rowOutrasContribuicoes){
        try {
            if($rowContaContabil[0]['__is_new']){
                $this->validaContaContabil($rowContaContabil[0], $rowOutrasContribuicoes);
            }
            $dataSet = new DataSet('outreccontctbl', $rowContaContabil);
            $this->dataSourceManager->persist($dataSet);
        }catch (\Exception $e) {
            throw $e;
        }
    }

    public function salvarBenfeitoria($row){
        try {
            if($row[0]['__is_new']){
               $row[0]['NRSEQBENFEITORIA'] = $this->geraNovoCodigo('BENFEITORIA',$this->nrorg);
            }
            $this->validaBenfeitorias($row[0]);
            $dataSet = new DataSet('benfeitoria', $row);
            $this->dataSourceManager->persist($dataSet);
        }catch (\Exception $e) {
            throw $e;
        }
    }

    public function deletarOutrasContribuicoes($row){
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();

            $this->factory->deleteContaContabilOutrasContribuicoes($this->nrorg, $row['CDEMPRESA'], $row['CDOUTRECCONT']);                        $this->entityManager->flush();

            $dataSet = new DataSet('outreccont', [$row]);
            $this->dataSourceManager->delete($dataSet);
            $this->entityManager->flush();

            $connection->commit();
        }catch (\Exception $e) {
            throw $e;
        }
    }

    public function validaAliquotaOutrasContribuicoes($row, $aliquota,$cdimposto){
        try {
            if($cdimposto=='50'){
                $mensagem = self::ALIQUOTA_PIS_INVALIDA;
            }else if($cdimposto=='60'){
                $mensagem = self::ALIQUOTA_COFINS_INVALIDA;
            }

            $aliquotaPis = $this->factory->validaAliquota($row['CDSITUTRIBPICO'], $aliquota, $cdimposto);
            if(empty($aliquotaPis)){
                throw new \Exception($mensagem, 1);
            }
        }catch (\Exception $e) {
            throw $e;
        }
    }

    public function validaNaturezaReceitaOutrasContribuicoes($row){
        try {
            if( in_array($row['CDSITUTRIBPICO'],['04','05','06','07','08','09']) ){
                if(!$row['CDNATUREC'] ){
                    throw new \Exception(self::NATUREZA_RECEITA, 1);
                }
            }else{
                if($row['CDNATUREC']){
                    throw new \Exception(self::NATUREZA_RECEITA_INVALIDA, 1);
                }
            }
        }catch (\Exception $e) {
            throw $e;
        }
    }

    public function validaDatas ($row){
        try {
            $resul = $this->factory->validaIntervaloDatas ($row['CDSITUTRIBPICO'], $row['CDEMPRESA'], $row['CDNATUREC'], $row['DTINIVALID'], $row['VRALIQCOFINS'], $row['IDOPERPICO'], $row['CDOUTRECCONT'], $row['DTFIMVALID'], $row['VRALIQPIS'] );
            if(!empty($resul)){
                if( $resul[0]['ERRO'] ){
                    if($resul[0]['ERRO']=='DtFim_Inf'){
                        throw new \Exception(self::DTFIM_INF, 1);
                    }else if($resul[0]['ERRO']=='DtInt_Inv'){
                        throw new \Exception(self::DTINT_INV, 1);
                    }else if($resul[0]['ERRO']=='DtFimOld_Inf'){
                        throw new \Exception(self::DTFIMOLD_INF, 1);
                    }
                }
            }
        }catch (\Exception $e) {
            throw $e;
        }
    }

    public function validaContaContabil ($rowContaContabil, $rowOutrasContribuicoes){
        try {
            $resul = $this->factory->validaContaContabil ($rowContaContabil['CDEMPRESA'], $rowContaContabil['CDCONTCTBL'], $rowContaContabil['CDNVCONTCTBL'], $rowOutrasContribuicoes['DTINIVALID'], $rowOutrasContribuicoes['DTFIMVALID']);
            if(!empty($resul)){
                if( $resul[0]['IDERRO'] ){
                    if($resul[0]['IDERRO']=='S'){
                        throw new \Exception("Existe nível superior cadastrado para o CST NF-e ".$resul[0]['CDSITUTRIBPICO']." no período de validade informado.", 1);
                    }else if($resul[0]['IDERRO']=='E'){
                        throw new \Exception("Conta contábil já cadastrado para o CST NF-e ".$resul[0]['CDSITUTRIBPICO']." no período de validade informado.", 1);
                    }else if($resul[0]['IDERRO']=='I'){
                        throw new \Exception("Existe nível inferior cadastrado para o CST NF-e ".$resul[0]['CDSITUTRIBPICO']." no período de validade informado.", 1);
                    }
                }
            }
        }catch (\Exception $e) {
            throw $e;
        }
    }

    public function validaBenfeitorias($row){
        try {
            $resul = $this->factory->validaBenfeitorias ($row['CDEMPRESA'], $row['CDCONTCTBL'], $row['CDNVCONTCTBL'], $row['NRSEQBENFEITORIA']);
            if(!empty($resul)){
                if( $resul[0]['IDERRO'] ){
                    if($resul[0]['IDERRO']=='S'){
                        throw new \Exception(self::CONTA_CONTABIL_SUPERIOR, 1);
                    }else if($resul[0]['IDERRO']=='E'){
                        throw new \Exception(self::CONTA_CONTABIL_CADASTRADA, 1);
                    }else if($resul[0]['IDERRO']=='I'){
                        throw new \Exception(self::CONTA_CONTABIL_INFERIOR, 1);
                    }
                }
            }
        }catch (\Exception $e) {
            throw $e;
        }
    }

}