<?php
namespace Teknisa\FIS\Services;

use Zeedhi\Framework\DataSource\FilterCriteria;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;


class Ger00250ParametrizacaoInscricaoEstadualService{
    protected $environment;
    protected $dataSourceManager;
    protected $entityManager;
    protected $newCode;

    public function __construct(Environment $environment, \Doctrine\ORM\EntityManager $entityManager ,$dataSourceManager,NewCode $newCode){
        $this->environment = $environment;
        $this->entityManager = $entityManager;
        $this->dataSourceManager = $dataSourceManager;
        $this->newCode = $newCode;
    }

    public function buscar($condicao){
        return $this->dataSourceManager->findBy($condicao);
    }

    public function geraNovoCodigo($tableName, $nrorg, $interval, $colSize){
        return $this->newCode->getCodeByTableName($tableName, $nrorg, $interval, $colSize);
    }

    public function salvarSubstitutoTributario($row){
        try {
            $conditions = $this->condicaoSubstituto($row);
            $resultadoBanco = $this->realizaConsultaBanco($conditions,'inscrestadst');
            if($row['__is_new']){
                if(empty($resultadoBanco->getRows())){
                    $conditionsAdicionar = $this->condicaoSubstitutoAdicionar($row);
                    $resultado = $this->realizaConsultaBanco($conditionsAdicionar,'inscrestadst');
                    if(empty($resultado->getRows() ) ){
                        $operador = $this->environment->getCdOperador();
                        $nrorg = $this->environment->getNrorgTrab();
                        $currentTime = $this->environment->getCurrentTime()->format('d/m/Y');
                        $row->set('IDATIVO','S');
                        $row->set('NRORG',$nrorg);
                        $row->set('NRORGULTATU',$nrorg);
                        $row->set('DTINCLUSAO',$currentTime);
                        $row->set('DTULTATU',$currentTime);
                        $row->set('CDOPERINCLUSAO',$operador);
                        $row->set('CDOPERULTATU',$operador);
                        $row->set('NRORGINCLUSAO',$nrorg);
                        $array[] = $row;
                        $dataSet = new DataSet('inscrestadst_completa', $array);
                        $this->dataSourceManager->persist($dataSet);
                        $this->entityManager->flush();
                    }
                    else{
                        throw new \Exception('Código já cadastrado.');
                    }
                }
                else{
                    throw new \Exception('Inscrição Estadual já cadastrada para o estado informado.');
                }
            }
            else{
                if(empty($resultadoBanco->getRows())){
                    $array[] = $row;
                    $dataSet = new DataSet('inscrestadst_completa', $array);
                    $this->dataSourceManager->persist($dataSet);
                    $this->entityManager->flush();
                }
                else{
                    $resul = $resultadoBanco->getRows()[0];
                    if($resul['CDINSCESTAST']== $row['CDINSCESTAST']){
                        $array[] = $row;
                        $dataSet = new DataSet('inscrestadst_completa', $array);
                        $this->dataSourceManager->persist($dataSet);
                        $this->entityManager->flush();
                    }else{
                        throw new \Exception('Inscrição Estadual já cadastrada para o estado informado.');
                    }
                }
            }
        }catch (\Exception $e) {
            throw $e;
        }
    }


    public function preparRowDiferencialAliquota($row){
        try {
            $conditions  = $this->condicaoDiferencialAliquota($row);
            $resultadoBanco = $this->realizaConsultaBanco($conditions,'paramestsped');
            //verifica se ja existe registro
            if(empty( $resultadoBanco->getRows() )){
                $row->set('__is_new',true);
            }else{
                $row->set('__is_new',false);
            }
            return $row;
        }catch (\Exception $e) {
            throw $e;
        }
    }

    public function salvarDiferencialAliquota($row){
        try {
            $dataSet = new DataSet('paramestsped', $row);
            $this->dataSourceManager->persist($dataSet);
            $this->entityManager->flush();
        }catch (\Exception $e) {
            throw $e;
        }
    }

    public function salvarMensagem($row){
        try {
            if($row['__is_new']){
                $conditions = $this->condicaoMensagem($row);
                $resultadoBanco = $this->realizaConsultaBanco($conditions,'mensinscrestad');
                if(empty($resultadoBanco->getRows())){
                    $operador = $this->environment->getCdOperador();
                    $nrorg = $this->environment->getNrorgTrab();
                    $currentTime = $this->environment->getCurrentTime()->format('d/m/Y');
                    $novoCodigoMensagem = $this->geraNovoCodigo('MENSINSCRESTAD',$this->environment->getNrOrgTrab(), 1, 5);
                    $row->set('IDATIVO','S');
                    $row->set('NRORG',$nrorg);
                    $row->set('NRORGULTATU',$nrorg);
                    $row->set('DTINCLUSAO',$currentTime);
                    $row->set('DTULTATU',$currentTime);
                    $row->set('CDOPERINCLUSAO',$operador);
                    $row->set('CDOPERULTATU',$operador);
                    $row->set('NRORGINCLUSAO',$nrorg);
                    $row->set('NRMENSINSCRESTA',$novoCodigoMensagem);
                    $array[] = $row;
                    $dataSet = new DataSet('mensinscrestad', $array);
                    $this->dataSourceManager->persist($dataSet);
                    $this->entityManager->flush();
                }
                else{
                    throw new \Exception('Impossível Adicionar. Registro já existe.');
                }
            }else{
                $array[] = $row;
                $dataSet = new DataSet('mensinscrestad', $array);
                $this->dataSourceManager->persist($dataSet);
                $this->entityManager->flush();
            }
        }catch (\Exception $e) {
            throw $e;
        }
    }

    public function salvarMensagemSubstituto($row){
        try {
            if($row['__is_new']){
                $conditions = $this->condicaoMensagemSubstituto($row);
                $resultadoBanco = $this->realizaConsultaBanco($conditions,'mensinscestast');
                if(empty($resultadoBanco->getRows())){
                    $operador = $this->environment->getCdOperador();
                    $nrorg = $this->environment->getNrorgTrab();
                    $currentTime = $this->environment->getCurrentTime()->format('d/m/Y');
                    $novoCodigoMensagem = $this->geraNovoCodigo('MENSINSCESTAST',$this->environment->getNrOrgTrab(), 1, 5);
                    $row->set('IDATIVO','S');
                    $row->set('NRORG',$nrorg);
                    $row->set('NRORGULTATU',$nrorg);
                    $row->set('DTINCLUSAO',$currentTime);
                    $row->set('DTULTATU',$currentTime);
                    $row->set('CDOPERINCLUSAO',$operador);
                    $row->set('CDOPERULTATU',$operador);
                    $row->set('NRORGINCLUSAO',$nrorg);
                    $row->set('NRMENSINSCESTAST',$novoCodigoMensagem);

                    $array[] = $row;
                    $dataSet = new DataSet('mensinscestast', $array);
                    $this->dataSourceManager->persist($dataSet);
                    $this->entityManager->flush();
                }
                else{
                    throw new \Exception('Impossível Adicionar. Registro já existe.');
                }
            }else{
                $array[] = $row;
                $dataSet = new DataSet('mensinscestast', $array);
                $this->dataSourceManager->persist($dataSet);
                $this->entityManager->flush();
            }
        }catch (\Exception $e) {
            throw $e;
        }
    }

    public function condicaoDiferencialAliquota($row) {
        $conditions = array(
            array(
                "value" => $this->environment->getNrOrgTrab(),
                "operator" => "=",
                "columnName" => "NRORG"
            ),
            array(
                "value" => $row['CDEMPRESA'],
                "operator" => "=",
                "columnName" => "CDEMPRESA"
            ),
            array(
                "value" => $row['CDINSCESTA'],
                "operator" => "=",
                "columnName" => "CDINSCESTA"
            )

        );
        return $conditions;
    }

    public function condicaoSubstituto($row) {
        $conditions = array(
            array(
                "value" => $this->environment->getNrOrgTrab(),
                "operator" => "=",
                "columnName" => "NRORG"
            ),
            array(
                "value" => $row['SGESTADO'],
                "operator" => "=",
                "columnName" => "SGESTADO"
            ),
            array(
                "value" => $row['CDEMPRESA'],
                "operator" => "=",
                "columnName" => "CDEMPRESA"
            ),
            array(
                "value" => $row['CDPAIS'],
                "operator" => "=",
                "columnName" => "CDPAIS"
            ),
            array(
                "value" => $row['CDINSCESTA'],
                "operator" => "=",
                "columnName" => "CDINSCESTA"
            )
        );
        return $conditions;
    }

    public function condicaoSubstitutoAdicionar($row) {
        $conditions = array(
            array(
                "value" => $this->environment->getNrOrgTrab(),
                "operator" => "=",
                "columnName" => "NRORG"
            ),
            array(
                "value" => $row['CDINSCESTAST'],
                "operator" => "=",
                "columnName" => "CDINSCESTAST"
            ),
            array(
                "value" => $row['CDEMPRESA'],
                "operator" => "=",
                "columnName" => "CDEMPRESA"
            ),
            array(
                "value" => $row['CDINSCESTA'],
                "operator" => "=",
                "columnName" => "CDINSCESTA"
            )
        );
        return $conditions;
    }

    public function condicaoMensagem($row) {
        $conditions = array(
            array(
                "value" => $this->environment->getNrOrgTrab(),
                "operator" => "=",
                "columnName" => "NRORG"
            ),
            array(
                "value" => $row['CDMENSAGEM'],
                "operator" => "=",
                "columnName" => "CDMENSAGEM"
            ),
            array(
                "value" => $row['CDEMPRESA'],
                "operator" => "=",
                "columnName" => "CDEMPRESA"
            ),
            array(
                "value" => $row['CDINSCESTA'],
                "operator" => "=",
                "columnName" => "CDINSCESTA"
            )
        );
        return $conditions;
    }

    public function condicaoMensagemSubstituto($row) {
        $conditions = array(
            array(
                "value" => $this->environment->getNrOrgTrab(),
                "operator" => "=",
                "columnName" => "NRORG"
            ),
            array(
                "value" => $row['CDMENSAGEM'],
                "operator" => "=",
                "columnName" => "CDMENSAGEM"
            ),
            array(
                "value" => $row['CDEMPRESA'],
                "operator" => "=",
                "columnName" => "CDEMPRESA"
            ),
            array(
                "value" => $row['CDINSCESTA'],
                "operator" => "=",
                "columnName" => "CDINSCESTA"
            ),
            array(
                "value" => $row['CDINSCESTAST'],
                "operator" => "=",
                "columnName" => "CDINSCESTAST"
            )
        );
        return $conditions;
  }

    public function realizaConsultaBanco($conditions,$dataSourceName) {
        $filterCriteria = new FilterCriteria($dataSourceName,$conditions);
        $dataSet = $this->buscar($filterCriteria);
        return $dataSet;
    }

}